@@ -37,15 +37,14 @@
 
 #define GSD_DBUS_NAME         "org.gnome.SettingsDaemon"
 
-#define DEFAULT_GCONF_PREFIX "/apps/gnome_settings_daemon/plugins"
-#define GCONF_PREFIX_ENV     "GNOME_SETTINGS_DAEMON_GCONF_PREFIX"
-
 static char      *gconf_prefix = NULL;
 static gboolean   no_daemon    = TRUE;
+static gboolean   debug        = TRUE;
 
 static GOptionEntry entries[] = {
+        {"debug", 0, 0, G_OPTION_ARG_NONE, &debug, "Enable debugging code", NULL },
         {"no-daemon", 0, 0, G_OPTION_ARG_NONE, &no_daemon, N_("Don't become a daemon"), NULL },
-        {"gconf-prefix", 0, 0, G_OPTION_ARG_STRING, &gconf_prefix, "GConf prefix from which to load plugin settings", NULL},
+        {"gconf-prefix", 0, 0, G_OPTION_ARG_STRING, &gconf_prefix, N_("GConf prefix from which to load plugin settings"), NULL},
         {NULL}
 };
 
@@ -167,6 +166,24 @@ bus_register (void)
         return ret;
 }
 
+static void
+gsd_log_default_handler (const gchar   *log_domain,
+                         GLogLevelFlags log_level,
+                         const gchar   *message,
+                         gpointer       unused_data)
+{
+        /* filter out DEBUG messages if debug isn't set */
+        if ((log_level & G_LOG_LEVEL_MASK) == G_LOG_LEVEL_DEBUG
+            && ! debug) {
+                return;
+        }
+
+        g_log_default_handler (log_domain,
+                               log_level,
+                               message,
+                               unused_data);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -197,12 +214,7 @@ main (int argc, char *argv[])
                 exit (1);
         }
 
-        if (gconf_prefix == NULL) {
-                gconf_prefix = g_strdup (g_getenv (GCONF_PREFIX_ENV));
-                if (gconf_prefix == NULL) {
-                        gconf_prefix = g_strdup (DEFAULT_GCONF_PREFIX);
-                }
-        }
+        g_log_set_default_handler (gsd_log_default_handler, NULL);
 
         if (! no_daemon && daemon (0, 0)) {
                 g_error ("Could not daemonize: %s", g_strerror (errno));
@@ -212,19 +224,35 @@ main (int argc, char *argv[])
                 goto out;
         }
 
-        program = gnome_program_init(
-                PACKAGE, VERSION, LIBGNOME_MODULE,
-                argc, argv, GNOME_PARAM_NONE);
+        program = gnome_program_init (PACKAGE,
+                                      VERSION,
+                                      LIBGNOME_MODULE,
+                                      argc,
+                                      argv,
+                                      GNOME_PARAM_NONE);
 
-        manager = gnome_settings_manager_new (gconf_prefix);
-
-        res = gnome_settings_manager_start (manager, &error);
-        if (! res) {
-                g_warning ("Unable to start: %s", error->message);
-                g_error_free (error);
+        manager = gnome_settings_manager_new ();
+        if (manager == NULL) {
+                g_warning ("Unable to register object");
                 goto out;
         }
 
+        /* If we aren't started by dbus then load the plugins
+           automatically.  Otherwise, wait for an Awake etc. */
+        if (g_getenv ("DBUS_STARTER_BUS_TYPE") == NULL) {
+                error = NULL;
+                if (gconf_prefix != NULL) {
+                        res = gnome_settings_manager_start_with_settings_prefix (manager, gconf_prefix, &error);
+                } else {
+                        res = gnome_settings_manager_start (manager, &error);
+                }
+                if (! res) {
+                        g_warning ("Unable to start: %s", error->message);
+                        g_error_free (error);
+                        goto out;
+                }
+        }
+
         gtk_main ();
 
         g_free (gconf_prefix);
